use std::{
    collections::BTreeMap,
    time::{SystemTime, UNIX_EPOCH},
};

use hmac::{Hmac, Mac};
use rand::RngCore;
use sha2::Sha256;

fn unix_time() -> u64 {
    SystemTime::now()
        .duration_since(UNIX_EPOCH)
        .unwrap()
        .as_secs()
}

pub enum LobbyStatus {
    Alive,
    Inactive,
}

pub struct Lobby {
    last_heartbeat: u64,
    status: LobbyStatus,
}

impl Lobby {
    fn send_heartbeat(&mut self) {
        self.last_heartbeat = unix_time();
    }
}

pub struct LobbyManager {
    heartbeat_timeout: u64,
    server_key: Hmac<Sha256>,
    lobbies: BTreeMap<String, Lobby>,
}

impl LobbyManager {
    fn new() -> Self {
        let mut server_key_token: [u8; 64] = [0; 64];
        {
            rand::rng().fill_bytes(&mut server_key_token);
        }

        // TODO: Handle this expect better
        let server_key: Hmac<Sha256> = Hmac::new_from_slice(&server_key_token).expect("bad token?");

        Self {
            heartbeat_timeout: 30,
            server_key,
            lobbies: BTreeMap::new(),
        }
    }

    fn create_lobby(&mut self) {
        let lobby = Lobby {
            last_heartbeat: unix_time(),
            status: LobbyStatus::Alive,
        };

        self.lobbies.insert(uuid::Uuid::new_v4().to_string(), lobby);
    }

    fn tick(&mut self) {
        for lobby in &mut self.lobbies {
            if lobby.1.last_heartbeat > unix_time() + self.heartbeat_timeout {
                match lobby.1.status {
                    LobbyStatus::Alive => lobby.1.status = LobbyStatus::Inactive,
                    _ => {}
                }
            }
        }
    }

    fn cull_lobbies(&mut self) -> Result<usize, ()> {
        let mut key_list: Vec<String> = Vec::new();

        {
            for lobby in &self.lobbies {
                match lobby.1.status {
                    LobbyStatus::Inactive => key_list.push(lobby.0.clone()),
                    _ => {}
                }
            }
        }
        {
            for key in &key_list {
                self.lobbies.remove(key);
            }
        }

        Ok(key_list.len())
    }
}
